//Program to analyse frequency data from a neutral marker experiment, dealing with the 0 mutant case.  Fits start frequencies to the
//observed marker frequencies.

#include <iostream>
#include <vector>

using namespace std;

#include "shared_new.h"

vector<m_pop> mpops;
vector<m_pop> new_mpops;

int main(int argc, char *argv[]){
	
	//Set up random number generator
	int seed=atoi(argv[1]);
	gsl_rng_env_setup();
	gsl_rng *rgen = gsl_rng_alloc (gsl_rng_taus);
	gsl_rng_set (rgen, seed);

	//Initial parameters
	int n_mpops=2;  //Number of marker populations
	int sample_stop=1000;  //Limit on the number of generations sampled
	
	vector<int> real_times;
	vector<d_vec> real_counts;
	int last_gen=0;

	//Read in marker population data from file specified in argv[2]
	int quiet=0;
	int max_pop_size=0;
	const char* name=argv[2];
	SamplePopulationData (name,n_mpops,last_gen,sample_stop,quiet,max_pop_size,real_times,real_counts);
	
	//General setup
	int status;
	double size;

	//Optimisation of the selection coefficients using the GSL minimiser
		
	double last_fit=10000000000;
	
	vector<double> start_freq;
	vector<double> start_freq_search;
	for (int i=0;i<n_mpops;i++){
		start_freq.push_back(0);
		start_freq_search.push_back(0);
	}

	int n_mut=0;
	//Begin optimisation process
	size_t iter=0; //GSL iteration number
		
	gsl_vector *x = gsl_vector_calloc((2*n_mut)+(n_mpops-1)); //Variables for GSL optimisation
	 
	for (int c=0;c<n_mpops-1;c++){
		gsl_vector_set(x,(2*n_mut)+c,1/(n_mpops+0.));
	}
	
	//Define the optimisation parameters
	int p_size = real_times.size()+(n_mpops*real_counts.size()) + n_mut + 9;
	cout << "Params size is " << p_size << "\n";		
	double *params;
	params=(double *)calloc(p_size,sizeof(double));

	SetParamsNoMut (params, n_mpops,last_gen,seed,real_counts,real_times);
	
	//Define the optimisation function
	gsl_multimin_function my_func;
	my_func.n=n_mpops-1;
	my_func.f=&minimise_freqs_nomut;
	my_func.params= params;
			
	gsl_multimin_fminimizer *s;
	const gsl_multimin_fminimizer_type *T=gsl_multimin_fminimizer_nmsimplex;
	s=gsl_multimin_fminimizer_alloc(T,(2*n_mut)+(n_mpops-1));

	gsl_vector* ss=gsl_vector_alloc((2*n_mut)+(n_mpops-1));
	for (int i=0;i<(2*n_mut)+(n_mpops-1);i++){
		if ((i<(2*n_mut))&&(i%2==1)){
			gsl_vector_set(ss,i,1);
		} else {
			gsl_vector_set(ss,i,0.1);
		}
	}
	
	gsl_multimin_fminimizer_set (s,&my_func,x,ss);
	
	do {
		iter++;
		status=gsl_multimin_fminimizer_iterate(s);
		if (status) {break;}
		
		size=gsl_multimin_fminimizer_size (s);
		status = gsl_multimin_test_size(size, 1e-4);
		
		if (status == GSL_SUCCESS) {
		}

	} while (status==GSL_CONTINUE && iter<100);
		
	last_fit = s->fval;	

	for (int c=0;c<n_mpops-1;c++){
		start_freq_search[c]=gsl_vector_get(s->x,c);
	}
	
	start_freq_search[n_mpops-1]=1;
	for (int c=0;c<n_mpops-1;c++){
		cout << "Population " << c << " frequency " << start_freq_search[c] << "\n";
		start_freq_search[n_mpops-1]=start_freq_search[n_mpops-1]-start_freq_search[c];
	}
	cout << "Population " << n_mpops-1 << " frequency " <<  start_freq_search[n_mpops-1] << "\n";
	cout << last_fit << "\n";
	gsl_vector_free(x);
	gsl_vector_free(ss);
	gsl_multimin_fminimizer_free (s);
	
	return EXIT_SUCCESS;
}
	
	
