
#include <iostream>
#include <iomanip>
#include <fstream>
#include <cstring>
#include <vector>
#include <string>
#include <algorithm>
#include <map>


using namespace std;

struct d_vec {
	vector<double> count;
	int x_sum;
};

struct mutant {
	double time;
	int mpop;
	double selection;
	double z_init;
	int order;
	int c_index;
};

struct m_pop {
	vector<double> individuals;
	vector<double> fitness;
};

struct mut_data {
	vector<mutant> data;
	vector<double> sf;
	double logL;
	double fm_size;
};

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>
#include <gsl/gsl_sf_hyperg.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_histogram.h>
#include <gsl/gsl_statistics_double.h>
#include <gsl/gsl_permutation.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_multimin.h>


void SamplePopulationData (const char* name, int n_mpops, int& last_gen, int sample_stop, int quiet, int& max_pop_size, vector<int>& real_times, vector<d_vec>& real_counts);
void GenerateInitialMutants (int n_mut, int n_mpops, int last_gen, gsl_rng *rgen, vector<mutant>& mutations);

void CheckLastLikelihood (int n_mut, int n_mpops, int& fail, double step,  double& best_fit, double& last_fit, double& fm_size, double fm_ss, vector<int>& index, vector<double>& start_freq, vector<double>& start_freq_search, vector<mutant>& mutations, vector<mutant>& mutations_search, gsl_vector *mut_info, int standing, int systematic);

void SystematicSearchParameters(int run, vector<int>& change);

void FirstOptimisationStep (int it, int fail, int n_mut, int n_mpops, int last_gen, double& step, vector<mutant>& mutations, vector<mutant>& mutations_search, gsl_rng *rgen);
void SystematicSearchStep (int& finish, int ok, int it, int n_mut, int last_gen, vector<int>& index, vector<int>& change, vector<mutant>& mutations, vector<mutant>& mutations_search);

void SetXVector (int n_mut, int n_mpops, double fm_ss, vector<mutant>& mutations_search, gsl_vector *x, int verbose, int standing, int systematic);

void SetParamsNoMut (double *params, int n_mpops, int last_gen, int seed, vector<d_vec> real_counts, vector<int> real_times);
void GetParamsNoMut (double *p, double& logL, int &bk, int& n_mpops, int& last_gen, int& rss, vector<d_vec>& real_counts, vector<int>& real_times, vector<double>& start_freqs, const gsl_vector *x);
void Setq0NoMut (int n_mpops, vector<double>& q0, vector<double> start_freqs); 

void SetParams (double *params, int p_size, int n_mut,int n_mpops, int last_gen, int temp, int find_mean, int max_pop_size, int systematic, int standing, double new_mut_size, vector<d_vec> real_counts, vector<mutant> mutations_search, vector<int> real_times, vector<double> fact_store);
void GetParams(double *p, int& bk, int& n_mut, int& n_mpops, int& last_gen, int& ps, int& rss, double& new_mut_size, int& find_mean, int& max_pop_size, int& systematic, int& standing, double& fm_size, double& logL, vector<mutant>& mutations, vector<int>& real_times, vector<d_vec>& real_counts, vector<double>& fact_store, vector<double>& start_freqs, const gsl_vector *x);
void Setq0(int& bk, int n_mpops, int n_mut, int fm_size, int standing, double new_mut_size, double& logL, vector<double>& q0, vector<double> start_freqs, vector<mutant> mutations);

double FindLikelihood_new (int print_freq, int n_mut, int n_mpops, int last_gen, int find_mean, vector<d_vec> real_counts, vector<int> real_times, vector<double> q0, vector<mutant>& mutations, vector<double>& fact_store);
double FindLikelihood_nomut (int print_freq, int n_mpops, int last_gen, vector<d_vec> real_counts, vector<int> real_times, vector<double> q0, vector<double>& fact_store);

void FindZa (int n_mut, vector<mutant>& mutations);
void FindHaploPops (double& total_q, int t, int n_mpops, int n_mut, vector<double> q0, vector<mutant> mutations, vector<double>& h_pop);
void FindHaploPopsNoMut (double& total_q, int t, int n_mpops, vector<double> q0, vector<double>& h_pop);
void FindHaploFreqs (int n_mpops, int n_mut, double total_q, vector<double>& h_pop, vector<double>& h_freq);
void FindMarkerFreqs (int t, int print_freq, int n_mpops, int n_mut, ofstream& mf, vector<double> h_pop, vector<double> h_freq, vector<double>& m_freq, vector<mutant> mutations);
void FindMarkerFreqsNoMut (int t, int print_freq, int n_mpops, ofstream& mf, vector<double> h_freq, vector<double>& m_freq);

void LikelihoodCalc (int s, int& n_points, double& logL, vector<d_vec> real_counts, vector<double> fact_store, vector<double> m_freq);



double minimise_freqs_new (const gsl_vector *x, void *params);

void DoFinalOutput (int alt, double new_mut_size, double best_fit, double fm_size, int n_mut, int last_gen, int n_mpops, int find_mean, vector<mutant>& mutations, vector<d_vec>& real_counts, vector<int>& real_times, vector<double>& start_freq, vector<double> fact_store, int standing);

void GetTransferData (int& n_mut, vector<mutant>& mutations, int& n_mpops, vector<double>& start_freq, double& best_fit);

void WriteTransferData (int alt, int n_mut, int n_mpops, double best_fit, double fm_size, vector<double>& start_freq, vector<mutant>& mutations, int standing);
void WriteTransferDataFile (int reps, int n_mut, int n_mpops, vector<mut_data>& mutation_store, int standing);

void find_logfact(vector<double>& fact_store, int N);

void InitialSystematicOutput (int n_mut, vector<int>change, vector<int>& index);

double minimise_freqs_nomut (const gsl_vector *x, void *params);
